# Package: PENTATrainer2
# Version: 1.1.3
# Filename: PENTAtrainer2.praat
#
# Author: Santitham Prom-on, Yi Xu
# Last updated: 2 February 2017
#
# Objectives: 
# 1.	Annotate input speech files iteratively: pulse marking, pitch extraction, 
#		boundary identification and functional marking
# 2.	Convert user-defined TextGrid into the annotate_short format
# 3.	Learn underlying target using stochastic optimization (simulated annealing) 
#		and analysis-by-synthesis

## Macro ##
taskEdit = 1
taskResume = 2
taskCreate = 3
taskAdd = 4
taskRemove = 5
taskImport = 6
taskLearn = 7
taskSynthesize = 8
#

##### Input Module - Begin #####

#Select the working directory
directory$ = "."

#Check whether the selected directory contain .wav or .WAV files
Create Strings as file list... FileList 'directory$'/*.wav
numberOfFiles = Get number of strings
if !numberOfFiles
	select Strings FileList
	Remove
	Create Strings as file list... FileList 'directory$'/*.WAV
	numberOfFiles = Get number of strings
endif

#[Error checking] No files in the directory	
if numberOfFiles = 0
	#[EXIT_A01] Sound files not found
	exit Error Code A01: No sound files in the directory
else
	Write to raw text file... 'directory$'/FileList.txt
		more_file = 1
endif

Create Table with column names... f0check 'numberOfFiles' Name Flag

#Read annotation files
Create Strings as file list... annot_list 'directory$'/*.annotation_short
n_annot = Get number of strings
Remove

# Main option window
beginPause("PENTAtrainer - Tasks")
	comment("Select a task")
	optionMenu("Task:", 1)
	if numberOfFiles = n_annot
		#Fully annotated
		option("Edit annotation")
		option("Create new annotation")
		option("Add new function")
		option("Remove function")
		option("Import annotation")
		option("Learn")
		option("Synthesize")
		boolean("Process all files without pause",0)
		boolean("Edit F0 analysis options",0)
	elsif n_annot > 0 
		#Partially annotated
		option("Resume annotation")
		option("Create new annotation")
		process_all_files_without_pause = 0
		edit_F0_analysis_options = 0
	else
		#Not annotated
		option("Create new  annotation")
		option("Import annotation")
		boolean("Edit F0 analysis options",0)
		process_all_files_without_pause = 0
	endif
	boolean("Inspect pulse",0)
clicked = endPause("Cancel","Start",2,1)

#[EXIT_00] Choose to not Continue
if clicked = 1
	exit
endif

#Reassign task variable based on the annotation status
if numberOfFiles = n_annot
	#Fully annotated
	if task = 1
		task = taskEdit
	elsif task = 2
		task = taskCreate
	elsif task = 3
		task = taskAdd
	elsif task = 4
		task = taskRemove
	elsif task = 5
		task = taskImport
	elsif task = 6
		task = taskLearn
	else
		task = taskSynthesize
	endif
elsif n_annot > 0 
	#Partially annotated
	if task = 1
		task = taskResume
	else
		task = taskCreate
	endif
else	
	#Not annotated
	if task = 1
		task = taskCreate
	else
		task = taskImport
	endif
endif

# If users want to edit the F0 analysis option
if fileReadable("'directory$'/config.txt")
	##This directory has been annotated before
	
	Read from file... 'directory$'/config.txt
	c_nrows = Get number of rows
	c_ncols = Get number of columns
	
	#[EXIT_C01] Wrong config file format
	if c_nrows <> 7 and c_ncols <> 2
		exit Error Code C01: Wrong config file format
	endif
	
	#Read values from the existing config file
	c_np = Get value... 1 value
	c_minf0 = Get value... 2 value
	c_maxf0 = Get value... 3 value
	c_fsr = Get value... 4 value
	c_pl = Get value... 5 value
	c_fo = Get value... 6 value
	c_sit = Get value... 7 value
	Remove
	
	#Create a form with values read from the existing config file
	if edit_F0_analysis_options
		beginPause ("Edit F0 Analysis Options")
			integer("left F0 range (Hz)",'c_minf0')
			integer("right F0 range (Hz)", 'c_maxf0')
			integer("Number of point per interval",'c_np')
			integer("F0 sample rate (Hz)",'c_fsr')
			integer("Perturbation length (ms)",'c_pl')
			real("Final offset",'c_fo')
			boolean("Set initial time for normf0 to 0",'c_sit')
		clicked = endPause("Cancel","Continue",2,1)
		
		#[EXIT_00] Choose to not continue
		if clicked = 1
			exit
		endif
		
		npoints = number_of_point_per_interval
		minf0 = left_F0_range
		maxf0 = right_F0_range
	else
		#or pass the values to the working variables
		npoints = c_np
		minf0 = c_minf0
		maxf0 = c_maxf0
		f0_sample_rate = c_fsr
		perturbation_length = c_pl
		final_offset = c_fo
		set_initial_time_for_normf0_to_0 = c_sit
	endif
else
	##This directory has not been annotated before, create
	
	if edit_F0_analysis_options
		#Prompt the window to display analysis options
		beginPause ("Edit F0 Analysis Options")
			integer("left F0 range (Hz)",30)
			integer("right F0 range (Hz)", 400)
			integer("Number of point per interval",10)
			integer("F0 sample rate (Hz)",100)
			integer("Perturbation length (ms)",0)
			real("Final offset",0)
			boolean("Set initial time for normf0 to 0",1)
		clicked = endPause("Cancel","Continue",2,1)
		
		#[EXIT_00] Choose to not continue
		if clicked = 1
			exit
		endif
		
		npoints = number_of_point_per_interval
		minf0 = left_F0_range
		maxf0 = right_F0_range
	else
		#Edit F0 analysis option is not selected. Set all options to default values
		npoints = 10
		minf0 = 30
		maxf0 = 400
		f0_sample_rate = 100
		perturbation_length = 0
		final_offset = 0
		set_initial_time_for_normf0_to_0 = 1
	endif
endif

#Create and save the Table object to be saved with the input values
Create Table with column names... config 7 parameter value
Set string value... 1 parameter npoints
Set numeric value... 1 value 'npoints'

Set string value... 2 parameter minf0
Set numeric value... 2 value 'minf0'

Set string value... 3 parameter maxf0
Set numeric value... 3 value 'maxf0'

Set string value... 4 parameter f0_sample_rate
Set numeric value... 4 value 'f0_sample_rate'

Set string value... 5 parameter perturbation_length
Set numeric value... 5 value 'perturbation_length'

Set string value... 6 parameter final_offset
Set numeric value... 6 value 'final_offset'

Set string value... 7 parameter set_initial_time_for_normf0_to_0
Set numeric value... 7 value 'set_initial_time_for_normf0_to_0'

Save as tab-separated file... 'directory$'/config.txt
Remove

##### Input Module - End #####

##### Task Module - Begin #####
if task = taskEdit
	# Scenario: The annotation is completed. Users want to edit the existing annotation.

	#Select the starting file to annotate
	beginPause("Edit existing annotation")
		integer("Input File No (from 'n_annot')", 1)
	clicked = endPause("Cancel","Continue",2,1)
	
	#[EXIT_00] Choose to not continue
	if clicked = 1
		exit
	endif
	
	#[EXIT_A02] Error: Input File No must be less than or equal to 'n_annot'
	if input_File_No > n_annot
		exit Error Code A02: Input File No must be less than or equal to 'n_annot'
	endif
	
;	if not fileReadable("'directory$'/function_profile.txt")
		#Read from the first file in FileList.txt to fine name$
		select Strings FileList
		fileName$ = Get string... 1
		name$ = fileName$ - ".wav" - ".WAV"
		
		#Read from the name$.annotation_short to find the list of communicative functions
		Read from file... 'directory$'/'name$'.annotation_short
		n_func = Get number of tiers
		list_func$ = ""		
		func_name$ = Get tier name... 1
		list_func$ = func_name$
		if n_func > 1
			for i from 2 to n_func
				func_name$ = Get tier name... i
				list_func$ = list_func$ + " " + func_name$
			endfor
		endif
		Remove
		
		#Create a table containing the functional profile
		#   (1) list of communicative function
		#   [depreciated] (2) the file extension to import
		#   [depreciated] (3) the tier number to import
		#   [depreciation note: (2) and (3) are replaced by import.praat
		Create Table with column names... functionProfile 3 parameter value
		Set string value... 1 parameter list_func
		Set string value... 1 value 'list_func$'
		Set string value... 2 parameter copy_from
		Set string value... 2 value n/a
		Set string value... 3 parameter copy_tier
		Set numeric value... 3 value 0
		Save as tab-separated file... 'directory$'/function_profile.txt
		Remove
;	endif
	
elsif task = taskResume
	# Scenario: The annotation is not finished. Users want to resume the ongoing annotation

	beginPause("Do you want to resume current annotation?")
	opt = endPause("No","Yes",2,1);
	if opt == 2
		#Users choose to resume
		input_File_No = n_annot + 1
		if fileReadable("'directory$'/function_profile.txt")
			Read from file... 'directory$'/function_profile.txt
			c_nrows = Get number of rows
			c_ncols = Get number of columns
			#[EXIT_C02] Wrong function profile file format
			if c_nrows <> 3 and c_ncols <> 2
				exit Error Code C02: Wrong function profile file format
			endif
			list_func$ = Get value... 1 value
			copy_from$ = Get value... 2 value
			copy_tier = Get value... 3 value
			task = taskCreate
			Remove
		else
			#[EXIT C03] Cannot find function profile
			exit Error Code C03: Cannot find function profile
		endif
	else
		#Users do not choose to resume
		exit
	endif

elsif task = taskCreate
	# Scenario: Users want to create new annotation
	input_File_No = 1
	beginPause("Create new annotation")
		sentence("List of communicative functions", "Tone Focus Sentence")
	clicked = endPause("Cancel","Continue",2,1)
	
	#[EXIT_00] Choose to not continue
	if clicked = 1
		exit
	endif
	
	list_func$ = list_of_communicative_functions$
	copy_from$ = "" ;copy_boundary_from$
	copy_tier = 1 ;boundary_tier
	copy_content = 1 ;copy_tier_content
	
	### Error Checking
		#[EXIT_A03] No input communicative functions
		if list_func$ == ""
			exit Error Code A03: No input communicative functions
		endif
		
		if copy_from$ <> ""
			if not startsWith(copy_from$,".")
				copy_from$ = "." + copy_from$
			endif
			Create Strings as file list... b_list 'directory$'/*'copy_from$'
			n_boundary = Get number of strings
			
			#[EXIT_A04] Cannot find source boundary files to copy from
			if n_boundary = 0
				exit Error Code A04: Cannot find source boundary files to copy from
			endif
			
			#[EXIT_A05] Number of source boundary files must equal number of sound files
			if n_boundary <> numberOfFiles
				exit Error Code A05: Number of source boundary files must equal number of sound files
			endif
			
			select Strings b_list
			fileName$ = Get string... 1
			Read from file... 'directory$'/'fileName$'
			fullName$ = selected$()
			type$ = extractWord$ (fullName$, "")
			
			#[EXIT_A06] Wrong boundary file type (TextGrid is required)
			if type$ <> "TextGrid"
				exit Error Code A06: Wrong boundary file type. TextGrid is required instead 'type$' was found.
			else
				nTierBoundaryFile = Get number of tiers
				#[EXIT_A07] Selected tier to copy from exceed the number of existing tiers
				if copy_tier > nTierBoundaryFile
					exit Error Code A07: Selected tier to copy from exceed the number of existing tiers
				endif
				
				#[EXIT_A08] Selected tier to copy from must be more than zero
				if copy_tier < 1
					exit Error Code A08: Selected tier to copy from must be more than zero
				endif
			endif
			
						
			plus Strings b_list
			Remove
		endif
	### End of Error Checking
	
	Create Table with column names... functionProfile 3 parameter value
	Set string value... 1 parameter list_func
	Set string value... 1 value 'list_func$'
	Set string value... 2 parameter copy_from
	Set string value... 2 value 'copy_from$'
	Set string value... 3 parameter copy_tier
	Set numeric value... 3 value 'copy_tier'
	Save as tab-separated file... 'directory$'/function_profile.txt
	Remove
	
	#Ask if users want to replace all annotations
	if n_annot > 0
		beginPause("Annotation exist. Replace all?")
		replace_option = endPause("No","Yes",1)
		if replace_option = 1
			exit
		else
			#Delete existing file before begin annotation
			Create Strings as file list... annot_list 'directory$'/*.annotation_short
			n_annot = Get number of strings
			for i from 1 to n_annot
				select Strings annot_list
				fileName$ = Get string... i
				name$ = fileName$ - ".annotation_short"
				call deleteAuxFiles 'name$'
			endfor
			n_annot = 0
			Remove
		endif
	endif

elsif task == taskAdd
	# Scenario: Annotation is finished. Users want to add additional layer into the annotation
	input_File_No = 1
	beginPause("Add new function")
	sentence("Function to be added","")
	endPause("Continue",1)
	function_name$ = function_to_be_added$
	if function_name$ == ""
		#[EXIT_A10] Input function cannot be empty
		exit Error Code A10: Input function cannot be empty
	else
		if fileReadable("'directory$'/function_profile.txt")
			Read from file... 'directory$'/function_profile.txt
			c_nrows = Get number of rows
			c_ncols = Get number of columns
			#[EXIT_C02] Wrong function profile file format
			if c_nrows <> 3 and c_ncols <> 2
				exit Error Code C02: Wrong function profile file format
			endif
			list_func$ = Get value... 1 value
			list_func$ = list_func$ + " " + function_name$
			Set string value... 1 value 'list_func$'
			Save as tab-separated file... 'directory$'/function_profile.txt
			Remove
		else
			#[EXIT C03] Cannot find function profile
			exit Error Code C03: Cannot find function profile
		endif
	endif

elsif task == taskRemove
	# Scenario: Annotation is completed. Users want to remove one layer out

	# Prompt the Remove option window to users
	input_File_No = 1
	beginPause("Remove existing function")
	integer("Tier number",0)
	endPause("Continue",1)
	
	if fileReadable("'directory$'/function_profile.txt")
		Read from file... 'directory$'/function_profile.txt
		c_nrows = Get number of rows
		c_ncols = Get number of columns
		#[EXIT_C02] Wrong function profile file format
		if c_nrows <> 3 and c_ncols <> 2
			exit Error Code C02: Wrong function profile file format
		endif
		list_func$ = Get value... 1 value
		Create TextGrid... 0 1 "'list_func$'" 
		Rename... newListFunc
		Remove tier... tier_number
		new_list_func$ = ""
		new_nrows = Get number of tiers
		
		if new_nrows > 0
			func_name$ = Get tier name... 1
			new_list_func$ = func_name$
			if new_nrows > 1
				for i from 2 to new_nrows
					func_name$ = Get tier name... i
					new_list_func$ = new_list_func$ + " " + func_name$
				endfor
			endif
		else
			# EXIT_A11: Cannot remove function tier
			exit Error Code A11: Cannot remove function tier
		endif
		select TextGrid newListFunc
		Remove
		select Table function_profile
		Set string value... 1 value 'new_list_func$'
		Save as tab-separated file... 'directory$'/function_profile.txt
		Remove
	else
		#[EXIT C03] Cannot find function profile
		exit Error Code C03: Cannot find function profile
	endif
elsif task == taskImport
	call Import
	exit
elsif task == taskLearn
	call Learn
	exit
elsif task == taskSynthesize
printline taskSynthesize = 'taskSynthesize'
	call Synthesize
	if synthesis_task = 2
		select Table accuracy
		Set string value... numberOfFiles+1 Filename Mean
		Set numeric value... numberOfFiles+1 RMSE 'mean_rmse'
		Set numeric value... numberOfFiles+1 Correlation 'mean_corr'
		Save as tab-separated file... 'directory$'/accuracy_synthesis.txt
		plus Strings FileList
		Remove
		printline Synthesis completed!
		printline mean RMSE = 'mean_rmse'		mean Correlation = 'mean_corr'
		exit
	endif
endif	
##### Task Module - End #####

# for each file (from input_File_No to numberOfFiles), call Labeling 
for current_file from input_File_No to numberOfFiles
	select Strings FileList
	fileName$ = Get string... current_file
	name$ = fileName$ - ".wav" - ".WAV"
	call Labeling 'fileName$'
endfor

call ReportError

if cflag = 0
	printline === Error has occurred: Annotation incomplete ===
else
	printline === Annotation completed ===
endif

select Strings FileList
plus Table f0check
Remove
#===End of Main===
#####

##########Other Procedures##########

procedure Labeling file_name$
	#Prepare the files necessary to annotation
	Read from file... 'directory$'/'file_name$'
	if task = taskEdit or task = taskImport
		if fileReadable ("'directory$'/'name$'.annotation_short")
			Read from file... 'directory$'/'name$'.annotation_short
			
			#Duplicate and remove all boundaries
			Copy... 'name$'_annot
			select TextGrid 'name$'_annot
			annot_ntier = Get number of tiers
			for i from 1 to annot_ntier
				annot_ninterval = Get number of intervals... i
				n = annot_ninterval - 1
				for j from 1 to n
					Remove right boundary... i 1
				endfor
			endfor
			
			select TextGrid 'name$'
			ntiers = Get number of tiers
		else
			#[EXIT_A09] Annotation file 'name$' not found
			exit Error Code A09: Annotation file 'name$' not found
		endif
	elsif task = taskCreate
		select Sound 'name$'
		start_time = Get start time
		end_time = Get end time

		To TextGrid... "'list_func$'"
		ntiers = Get number of tiers
		
		Copy... 'name$'_annot
		select TextGrid 'name$'
			
		if copy_from$ != ""
			Read from file... 'directory$'/'name$''copy_from$'
			Rename... _boundary_
			nintervals = Get number of intervals... 'copy_tier'
			
			for j from 2 to nintervals
				select TextGrid _boundary_
				start = Get start point... 'copy_tier' j
				select TextGrid 'name$'
				Insert boundary... 1 start
				if copy_content > 0
					select TextGrid _boundary_
					labc$ = Get label of interval... 'copy_tier' j
					if labc$ <> ""
						select TextGrid 'name$'
						Set interval text... 1 j 'labc$'
					endif
				endif
			endfor
			
			select TextGrid _boundary_
			Remove
		endif
	elsif task = taskAdd
		if fileReadable ("'directory$'/'name$'.annotation_short")
			Read from file... 'directory$'/'name$'.annotation_short
			ntiers = Get number of tiers
			newtier_number = ntiers+1
			Insert interval tier... newtier_number 'function_name$'
			nintervals = Get number of intervals... 1
			
			for j from 2 to nintervals
				start = Get starting point... 1 j
				Insert boundary... newtier_number start
			endfor
			
			#Duplicate and remove all boundaries
			Copy... 'name$'_annot
			select TextGrid 'name$'_annot
			annot_ntier = Get number of tiers
			for i from 1 to annot_ntier
				annot_ninterval = Get number of intervals... i
				n = annot_ninterval - 1
				for j from 1 to n
					Remove right boundary... i 1
				endfor
			endfor
		else
			exit Annotation file 'name$' not found
		endif
	elsif task = taskRemove
		if tier_number > 0
			if fileReadable ("'directory$'/'name$'.annotation_short")
				Read from file... 'directory$'/'name$'.annotation_short
				Remove tier... tier_number
				ntiers = Get number of tiers
				
				#Duplicate and remove all boundaries
				Copy... 'name$'_annot
				select TextGrid 'name$'_annot
				annot_ntier = Get number of tiers
				for i from 1 to annot_ntier
					annot_ninterval = Get number of intervals... i
					n = annot_ninterval - 1
					for j from 1 to n
						Remove right boundary... i 1
					endfor
				endfor
			else
				exit Annotation file 'name$' not found
			endif
		endif
	endif
	
	if task <> taskRemove 
		if process_all_files_without_pause = 0
			select TextGrid 'name$'
			plus Sound 'name$'
			Edit
		 
		
			pulsefile$ = name$+".pulse"
			if fileReadable ("'directory$'/'pulsefile$'")
				Read from file... 'directory$'/'pulsefile$'
			else
				select Sound 'name$'
				To PointProcess (periodic, cc)... minf0 maxf0
				Rename... 'name$'
			endif
		
			if inspect_pulse
				plus Sound 'name$'
				Edit
			endif
		
			pause Click Continue when you have finished editing
		else
			printline Processing 'name$'
			pulsefile$ = name$+".pulse"
			if fileReadable ("'directory$'/'pulsefile$'")
				Read from file... 'directory$'/'pulsefile$'
			else
				select Sound 'name$'
				To PointProcess (periodic, cc)... minf0 maxf0
				Rename... 'name$'
			endif
		endif
	else
		pulsefile$ = name$+".pulse"
		if fileReadable ("'directory$'/'pulsefile$'")
			Read from file... 'directory$'/'pulsefile$'
		else
			select Sound 'name$'
			To PointProcess (periodic, cc)... minf0 maxf0
			Rename... 'name$'
		endif
	endif
	
	call CreateFullCombination 'name$'
	
	replot = 1
	call Save 'directory$' 'name$'
	
	select Sound 'name$'
	Remove
endproc


###Clean all files generated for the target file
procedure deleteAuxFiles name$
	if fileReadable("'directory$'/'name$'.actutimenormf0")
		deleteFile("'directory$'/'name$'.actutimenormf0")
	endif
	if fileReadable("'directory$'/'name$'.annotation")
		deleteFile("'directory$'/'name$'.annotation")
	endif
	if fileReadable("'directory$'/'name$'.annotation_short")
		deleteFile("'directory$'/'name$'.annotation_short")
	endif
	if fileReadable("'directory$'/'name$'.f0")
		deleteFile("'directory$'/'name$'.f0")
	endif
	if fileReadable("'directory$'/'name$'.f0velocity")
		deleteFile("'directory$'/'name$'.f0velocity")
	endif
	if fileReadable("'directory$'/'name$'.PitchTier")
		deleteFile("'directory$'/'name$'.PitchTier")
	endif
	if fileReadable("'directory$'/'name$'.pulse")
		deleteFile("'directory$'/'name$'.pulse")
	endif
	if fileReadable("'directory$'/'name$'.rawf0")
		deleteFile("'directory$'/'name$'.rawf0")
	endif
	if fileReadable("'directory$'/'name$'.samplef0")
		deleteFile("'directory$'/'name$'.samplef0")
	endif
	if fileReadable("'directory$'/'name$'.semitonef0")
		deleteFile("'directory$'/'name$'.semitonef0")
	endif
	if fileReadable("'directory$'/'name$'.timenormf0")
		deleteFile("'directory$'/'name$'.timenormf0")
	endif
endproc

###Save the auxiliary files related to name$###
procedure CreateFullCombination name$
	#Propagate the smallest tier to other tiers
	select TextGrid 'name$'
	ntier_template = Get number of tiers
	if ntier_template > 1
		#Search for the smallest tier
		s_tier = 1
		s_n = Get number of intervals... 1
		for i from 2 to ntier_template
			x_n = Get number of intervals... i
			if x_n > s_n
				s_tier = i
				s_n = x_n
			endif
		endfor                  
		
		#Insert boundaries to other tier
		for i from 1 to ntier_template
			for j from 1 to s_n
				select TextGrid 'name$'
				t_end = Get end point... s_tier j
				select TextGrid 'name$'_annot
				if (j<s_n)
					Insert boundary... i t_end
				endif	
			endfor
		endfor
		
		#Copy labels from .annotation_short
		for i from 1 to ntier_template
			for j from 1 to s_n
				select TextGrid 'name$'
				t_start = Get starting point... s_tier j
				t_end = Get end point... s_tier j
				t_mid = (t_start + t_end)/2
				x_int = Get interval at time... i t_mid
				label$ = Get label of interval... i x_int
				select TextGrid 'name$'_annot
				Set interval text... i j 'label$'
			endfor
		endfor
		
		#Delete intervals that should be voiceless
		for j from 1 to s_n
			select TextGrid 'name$'
			t_start = Get starting point... s_tier j
			t_end = Get end point... s_tier j
			t_mid = (t_start + t_end)/2
			x_int = Get interval at time... s_tier t_mid
			label$ = Get label of interval... s_tier x_int
			if label$ = ""
				select TextGrid 'name$'_annot
				for i from 1 to ntier_template
					Set interval text... i j 'label$'
				endfor
			endif				
		endfor
	elsif ntier_template = 1
		#Search for the smallest tier                  
		s_tier = 1
		s_n = Get number of intervals... 1
		
		#Insert boundaries to other tier
		for j from 1 to s_n
			select TextGrid 'name$'
			t_end = Get end point... s_tier j
			select TextGrid 'name$'_annot
			if (j<s_n)
				Insert boundary... 1 t_end
			endif
		endfor
		
		#Copy labels from .annotation_short
		for j from 1 to s_n
			select TextGrid 'name$'
			t_start = Get starting point... s_tier j
			t_end = Get end point... s_tier j
			t_mid = (t_start + t_end)/2
			x_int = Get interval at time... 1 t_mid
			label$ = Get label of interval... 1 x_int
			select TextGrid 'name$'_annot
			Set interval text... 1 j 'label$'
		endfor
		
		#Delete intervals that should be voiceless
		for j from 1 to s_n
			select TextGrid 'name$'
			t_start = Get starting point... s_tier j
			t_end = Get end point... s_tier j
			t_mid = (t_start + t_end)/2
			x_int = Get interval at time... s_tier t_mid
			label$ = Get label of interval... s_tier x_int
			if label$ = ""
				select TextGrid 'name$'_annot
				Set interval text... 1 j 'label$'
			endif				
		endfor
	endif
endproc

procedure Save directory$ name$
	select PointProcess 'name$'
	npulses = Get number of points
	for n from 2 to npulses
		time1 = Get time from index... n-1
		time2 = Get time from index... n
		if time2 = time1
			Remove point... n
		endif
	endfor
	Write to short text file... 'directory$'/'name$'.pulse
	maxperiod = 1/minf0
	To PitchTier... maxperiod
	Down to TableOfReal... Hertz
	Write to headerless spreadsheet file... 'directory$'/'name$'.rawf0
	Remove
	select PitchTier 'name$'
	call Trimf0
	Write to short text file... 'directory$'/'name$'.PitchTier
	Down to TableOfReal... Hertz
	Write to headerless spreadsheet file... 'directory$'/'name$'.f0
	Remove

	# Create sampled F0 and velocity with a constant sampling rate
	call Sampling
	if found_interval
		call Differentiation
		call Smooth "velocity" "smooth_velocity" 0.05 * f0_sample_rate
		select TableOfReal samplef0
		Write to headerless spreadsheet file... 'directory$'/'name$'.samplef0
		Remove
		select TableOfReal velocity
		Write to headerless spreadsheet file... 'directory$'/'name$'.f0velocity
		plus PitchTier smoothf0
		Remove
	endif

	# Create sampled F0 with a constant number per interval
	if found_interval
		call Normalization
		
		call CheckF0
		
		select TableOfReal normf0
		Write to headerless spreadsheet file... 'directory$'/'name$'.timenormf0
		select TableOfReal normactuf0
		Write to headerless spreadsheet file... 'directory$'/'name$'.actutimenormf0
		plus TableOfReal normf0
		plus TableOfReal normactuf0
		Remove
		
		select PitchTier smooth_velocity
		Remove

		select PitchTier 'name$'
		plus PitchTier velocity
		Remove
	endif
	
	select TextGrid 'name$'
	Write to short text file... 'directory$'/'name$'.annotation_short

	select TextGrid 'name$'_annot
	Write to short text file... 'directory$'/'name$'.annotation
	
	select TextGrid 'name$'_annot
	Down to Table... no 4 yes no
	Save as tab-separated file... 'directory$'/'name$'.intervals
	
	select TextGrid 'name$'
	plus TextGrid 'name$'_annot
	plus Table 'name$'_annot
	plus PointProcess 'name$'
	if found_interval
		plus PitchTier semitonef0
	endif
	Remove
endproc


procedure Trimf0
	maxbump = 0.01
	maxedge = 0.0
	maxgap = 0.033
	n = Get number of points
	
	first = Get value at index... 1
	second = Get value at index... 2
	penult = Get value at index... n-1
	last = Get value at index... n
	tfirst = Get time from index... 1
	tlast = Get time from index... n
	for k from 1 to 3
		call Trim
	endfor
	Remove point... 1
	Add point... tfirst second + (first-second) / 1000
	Remove point... n
	Add point... tlast penult + (last-penult) / 1000
endproc

procedure Trim
	for i from 2 to n-1
		tleft = Get time from index... i-1
		tmid = Get time from index... i
		tright = Get time from index... i+1
		gap1 = tmid - tleft
		gap2 = tright - tmid
		left = Get value at index... i-1
		mid = Get value at index... i
		right = Get value at index... i+1
		diff1 = mid - left
		diff2 = mid - right
		if diff1 > maxbump and diff2 > maxedge and gap1 < maxgap and gap2 < maxgap
		... or diff2 > maxbump and diff1 > maxedge and gap1 < maxgap and gap2 < maxgap
			Remove point... i
			Add point... tmid left+(tmid-tleft)/(tright-tleft)*(right-left)
		endif
		if diff1 > maxbump and gap2 >= maxgap
			Remove point... i
			Add point... tmid left + maxbump
		endif
		if diff2 > maxbump and gap1 >= maxgap
			Remove point... i
			Add point... tmid right + maxbump
		endif

		diff1 = left - mid
		diff2 = right - mid
		if diff1 > maxbump and diff2 > maxedge and gap1 < maxgap and gap2 < maxgap
		... or diff2 > maxbump and diff1 > maxedge and gap1 < maxgap and gap2 < maxgap
			Remove point... i
			Add point... tmid left+(tmid-tleft)/(tright-tleft)*(right-left)
		endif
		if diff1 > maxbump and gap2 >= maxgap
			Remove point... i
			Add point... tmid left - maxbump
		endif
		if diff2 > maxbump and gap1 >= maxgap
			Remove point... i
			Add point... tmid right - maxbump
		endif
	endfor
endproc

procedure Normalization
	select PitchTier 'name$'
	lasttime = Get finishing time
	
	# create empty PitchTiers named "normactutimef0" and "normactuf0"
	Create PitchTier... normactutimef0 0 lasttime
	Create TableOfReal... normf0 1 2
	Set column label (index)... 1 NormalizedTime
	Set column label (index)... 2 F0
	Create TableOfReal... normactuf0 1 2
	Set column label (index)... 1 ActualTime
	Set column label (index)... 2 F0
	
	select TextGrid 'name$'
	nintervals = Get number of intervals... 1
	interval = 0
	found_interval = 0
	nrows = 0
	# for each interval
	for m from 1 to nintervals
		select TextGrid 'name$'
		label$ = Get label of interval... 1 m
		if not label$ = ""
			start = Get starting point... 1 m
			end = Get end point... 1 m
			select PitchTier 'name$'
			index1 = Get high index from time... start
			index2 = Get low index from time... end
			if found_interval = 0
				found_interval = 1
				select PitchTier 'name$'
				firstime = start
			endif
			call Normalize
			interval = interval + 1
		endif
	endfor
	select TableOfReal normf0
	if nrows > 1
		Remove row (index)... nrows + 1
	endif
	select TableOfReal normactuf0
	if nrows > 1
		Remove row (index)... nrows + 1
	endif
	select PitchTier normactutimef0
	Remove
endproc

procedure Normalize
	duration = end - start

	for i from index1 to index2
		select PitchTier 'name$'
		time = Get time from index... i
		f0 = Get value at index... i
		select PitchTier normactutimef0
		Add point... (time-start)/duration+interval f0
	endfor
	
	for x from 1 to npoints
		normtime = x / npoints
		select PitchTier normactutimef0
		f0 = Get value at time... normtime+interval
		select TableOfReal normf0
		nrows = Get number of rows
		Set value... nrows 1 x+interval*npoints
		Set value... nrows 2 f0
		Set row label (index)... nrows 'label$'
		Insert row (index)... nrows + 1
		select TableOfReal normactuf0
		nrows = Get number of rows
		Set value... nrows 1 normtime*duration+start
		if set_initial_time_for_normf0_to_0
			Set value... nrows 1 normtime*duration+start-firstime
		endif
		Set value... nrows 2 f0
		Set row label (index)... nrows 'label$'
		Insert row (index)... nrows + 1
	endfor
endproc

# Sampling calculates the number of syllable from the sampling rate and samples F0 of the specified intervals
procedure Sampling
	# create an empty TableOfReal named "samplef0" with only the header
	select PitchTier 'name$'
	Create TableOfReal... samplef0 1 2
	Set column label (index)... 1 SampleTime
	Set column label (index)... 2 F0

	# extract the number of intervals from TextGrid annotation and initiate related variables
	select TextGrid 'name$'
	nintervals = Get number of intervals... 1
	nlabels = 0
	found_interval = 0
	nrows = 0
	
	# create an empty PitchTier named "samplef0" starting from sampleStart to sampleEnd
	sampleStart = Get starting point... 1 1
	sampleEnd = Get end point... 1 nintervals
	Create PitchTier... samplef0 sampleStart sampleEnd
	
	# for each interval
	for m from 1 to nintervals
		select TextGrid 'name$'
		label$ = Get label of interval... 1 m
		if not label$ = ""
			nlabels += 1
			# get the start and end time of the interval, calculate the duration, and set the found_interval flag to 1
			start = Get starting point... 1 m
			end = Get end point... 1 m
			duration = end - start
			if found_interval = 0
				found_interval = 1
			endif

			# calculate the number of samples for the inverval
			select PitchTier 'name$'
			nsamples = duration * f0_sample_rate + 1
			
			# for each sampled time
			for x from 0 to nsamples - 1
				# get sample_time and f0 at that time
				select PitchTier 'name$'
				sample_time = start + x/f0_sample_rate
				f0 = Get value at time... sample_time
				
				# insert sample_time and f0 to the PitchTier "samplef0" and to the TableOfReal "samplef0"
				select PitchTier samplef0
				Add point... sample_time f0
				select TableOfReal samplef0
				nrows = Get number of rows
				Set value... nrows 1 sample_time
				Set value... nrows 2 f0
				Set row label (index)... nrows 'label$'
				Insert row (index)... nrows + 1
			endfor
		endif
	endfor
	select TableOfReal samplef0
	if nrows > 1
		Remove row (index)... nrows + 1
	endif
endproc


procedure Differentiation
	select PitchTier samplef0
	Rename... semitonef0
	Formula... 12 * ln (self) / ln(2); semitone
	Down to TableOfReal... Hertz
	Set column label (index)... 1 time
	Set column label (index)... 2 f0 (st)

	Write to headerless spreadsheet file... 'directory$'/'name$'.semitonef0
	Remove
	call Smooth "semitonef0" "smoothf0" 0.05 * f0_sample_rate
	Create PitchTier... velocity sampleStart sampleEnd
	Create TableOfReal... velocity 1 2
	Set column label (index)... 1 SampleTime
	Set column label (index)... 2 Velocity(semitone/s)
	
	select TextGrid 'name$'
	nintervals = Get number of intervals... 1
	nrows = 0
	for m from 1 to nintervals
		select TextGrid 'name$'
		label$ = Get label of interval... 1 m
		if not label$ = ""
			start = Get starting point... 1 m
			end = Get end point... 1 m
			select PitchTier smoothf0
			index_first = Get high index from time... start
			index_last = Get low index from time... end
			for x from index_first to index_last - 1
				if x = index_first or x = index_last - 1
					x2 = x + 1
				else
					x2 = x + 2
				endif
				select PitchTier smoothf0
				f01 = Get value at index... x
				f02 = Get value at index... x2
				sampletime1 = Get time from index... x
				sampletime2 = Get time from index... x2
				velocity = (f02 - f01) / (sampletime2 - sampletime1)
				velocity_time = 0.5 * (sampletime1 + sampletime2)
				select PitchTier velocity
				Add point... velocity_time velocity
				select TableOfReal velocity
				nrows = Get number of rows
				Set value... nrows 1 velocity_time
				Set value... nrows 2 velocity
				Set row label (index)... nrows 'label$'
				Insert row (index)... nrows + 1
			endfor
		endif
	endfor
	select TableOfReal velocity
	if nrows > 1
		Remove row (index)... nrows + 1
	endif
endproc

procedure Means
	select Sound 'name$'
	To Intensity... 100 0 yes

	select TextGrid 'name$'
	nintervals = Get number of intervals... 1
	Create TableOfReal... means 1 16
	Set column label (index)... 1 maxf0
	Set column label (index)... 2 minf0
	Set column label (index)... 3 excursion_size
	Set column label (index)... 4 meanf0
	Set column label (index)... 5 finalf0
	Set column label (index)... 6 mean_intensity
	Set column label (index)... 7 duration
	Set column label (index)... 8 max_velocity
	Set column label (index)... 9 final_velocity
	Set column label (index)... 10 initialf0
	Set column label (index)... 11 target_slope
	Set column label (index)... 12 target_height
	Set column label (index)... 13 strength
	Set column label (index)... 14 duration
	Set column label (index)... 15 rmse
	Set column label (index)... 16 correlation

	interval = 0
	for mm from 1 to nintervals
		select TextGrid 'name$'
		label$ = Get label of interval... 1 mm
		if not label$ = ""
			interval = interval + 1
			start = Get starting point... 1 mm
			end = Get end point... 1 mm
			duration = 1000 * (end - start)
			start1 = start + perturbation_length/1000
			select TableOfReal means
			Set row label (index)... interval 'label$'
			select PitchTier 'name$'
			meanf0 = Get mean (points)... start1 end
			early_end = end + final_offset
			finalf0 = Get value at time... early_end
			To Pitch... 0.02 30 600
			max_f0 = Get maximum... start1 end Hertz Parabolic
			min_f0 = Get minimum... start1 end Hertz Parabolic
			excursionsize = hertzToSemitones(max_f0) - hertzToSemitones(min_f0)
			Remove
			select Intensity 'name$'
			intensity = Get mean... start end energy
			select PitchTier smooth_velocity
			final_velocity'interval' = Get value at time... early_end
			
			mid = 0.5 * (start + end)
			select PitchTier smooth_velocity
			index_first = Get high index from time... mid
			index_last = Get low index from time... end
			maxvelocity = 0
			for x from index_first to index_last
				v = Get value at index... x
				if abs(v) > abs(maxvelocity)
					maxvelocity = v
				endif
			endfor

			select TableOfReal means
			Set value... interval 1 max_f0
			Set value... interval 2 min_f0
			Set value... interval 3 excursionsize
			Set value... interval 4 meanf0
			Set value... interval 5 finalf0
			Set value... interval 6 intensity
			Set value... interval 7 duration
			Set value... interval 8 maxvelocity
			Set value... interval 9 final_velocity'interval'
			Insert row (index)... interval + 1
		endif
	endfor

	if found_interval
		call Target
		select Sound 'name$'
		To Manipulation... 0.01 75 600
		select PitchTier fittedf0
		Formula... exp(self*ln(2)/12)
		plus Manipulation 'name$'
		Replace pitch tier
		select TableOfReal fittedf0
		Write to headerless spreadsheet file... 'directory$'/'name$'.qTAf0
#		plus PitchTier fittedf0
		Remove
		select TableOfReal fittedvelocity
		Write to headerless spreadsheet file... 'directory$'/'name$'.qTAvelocity
		plus PitchTier fittedvelocity
		plus TableOfReal trainingf0
;		plus TableOfReal semitonef0
		Remove

		interval = 0
		for mm from 1 to nintervals
			select TextGrid 'name$'
			label$ = Get label of interval... 1 mm
			if not label$ = ""
				interval = interval + 1
				select TableOfReal means
				Set value... interval 10 xInitial
				Set value... interval 11 mBest'interval'
				Set value... interval 12 bBest'interval'
				Set value... interval 13 lambdaBest'interval'
				Set value... interval 14 duration'interval'
				Set value... interval 15 rmse'interval'
				Set value... interval 16 corr'interval'
			endif
		endfor
	endif
	select TableOfReal means
	nrows = Get number of rows
	if nrows > 1
		Remove row (index)... nrows
	endif
endproc

procedure Smooth curve_in$ curve_out$ width
	Create PitchTier... 'curve_out$' sampleStart sampleEnd
	
	for j from 1 to width							; make a triangular window of size = width 
		if j < width / 2 + 0.5
			weight'j' = j
		else 
			weight'j' = width - j + 1
		endif
	endfor

	select PitchTier 'curve_in$'
	smooth_end = Get number of points

	for i from 1 to width / 2						; smooth initial points: 0 to width/2 - 1 
	    n = 0.0
		smoothsample = 0.0
		select PitchTier 'curve_in$'
		sample_time = Get time from index... i
	    for j from 1 to width/2 + i					; window size = width/2 to width - 1
	    	j = 'j:0'
	    	rawsample = Get value at index... j
	    	index = width / 2 + j - i
	    	index = 'index:0'
			smoothsample += weight'index' * rawsample
			n += weight'index'
	    endfor
		smoothsample /= n
		if smoothsample != undefined
			select PitchTier 'curve_out$'
			Add point... sample_time smoothsample
		endif
	endfor
	
	for i from width/2 to smooth_end - width/2				; smooth from width/2 to end-width/2
		n = 0
		smoothsample = 0.0
		select PitchTier 'curve_in$'
		sample_time = Get time from index... i
		for j from 1 to width
	    	rawsample = Get value at index... i-width/2+j
			smoothsample += weight'j' * rawsample
			n += weight'j'
		endfor
		smoothsample /= n
		if smoothsample != undefined
			select PitchTier 'curve_out$'
			Add point... sample_time smoothsample
		endif
	endfor
	
	i = width/2
	while i > 0										; smooth final points: end - width/2 to end
		n = 0.0
		smoothsample = 0.0
		select PitchTier 'curve_in$'
		sample_time = Get time from index... smooth_end-i
		j = width/2 + i
		j = 'j:0'
		while j > 1									; window size = width - 1 to width/2 
	    	rawsample = Get value at index... smooth_end-j
	    	index = width/2+i-j + 1
	    	index = 'index:0'
			smoothsample += weight'index' * rawsample
			n += weight'index'
			j -= 1
		endwhile
		smoothsample /= n
		if smoothsample != undefined
			select PitchTier 'curve_out$'
			Add point... sample_time smoothsample
		endif
		i -= 1
	endwhile
endproc

procedure CheckF0
	flag = 1
	select TableOfReal normf0
	nrows = Get number of rows
	for i from 1 to nrows
		fx = Get value... i 2
		if fx = undefined
			flag = 0
		endif
	endfor
	
	select TableOfReal normactuf0
	nrows = Get number of rows
	for i from 1 to nrows
		fx = Get value... i 2
		if fx = undefined
			flag = 0
		endif
	endfor
		
	select Table f0check
	Set string value... current_file Name 'name$'
	Set numeric value... current_file Flag flag
endproc

procedure ReportError
	cflag = 1
	select Table f0check
	nrows = Get number of rows
	for i from 1 to nrows
		select Table f0check
		flag = Get value... i Flag
		name$ = Get value... i Name
		if flag = 0
			printline Error: undefined value at file: 'name$'
			cflag = 0
		endif
	endfor
endproc

procedure Import
# Objective: convert user-defined TextGrid into the annotate_short format
	select Strings FileList
	fileName$ = Get string... 1
	name$ = fileName$ - ".wav" - ".WAV"
	Create Strings as file list... extension 'directory$'/'name$'.*
	nExtensions = Get number of strings
	input_File_No = 1

	i=1
	while i<= nExtensions 
		select Strings extension
		ext$ = Get string... i
		nowarn Read from file... 'directory$'/'ext$'
		fullName$ = selected$()
		type$ = extractWord$(fullName$, "")
		Remove
		if type$ != "TextGrid"
			select Strings extension
			Remove string... i
			i = i - 1
			nExtensions = nExtensions -1
		endif
		i = i + 1
	endwhile

	#Prompt the user to select the TextGrid file extension
	beginPause("Import Annotation - File Extension")
		comment("Select file extension to import")
		optionMenu("File extension:", 1)
		for i from 1 to nExtensions
			select Strings extension
			ext$ = Get string... i
			pos = length(ext$) - rindex(ext$,".")
			ext$ = right$(ext$,pos)
			option(".'ext$'")
		endfor
	clicked = endPause("Cancel","Continue",2,1)

	#[EXIT_00] Choose to not Continue
	if clicked = 1
		exit
	endif

	#Extract the file extension
	select Strings extension
	ext$ = Get string... file_extension
	pos = length(ext$) - rindex(ext$,".")
	ext$ = right$(ext$,pos)

	#Prompt the user to select the tier name
	select Strings FileList
	fileName$ = Get string... 1
	name$ = fileName$ - ".wav" - ".WAV"
	Read from file... 'directory$'/'name$'.'ext$'
	ntier = Get number of tiers
	beginPause("Import Annotation - Tier")
		comment("Mark the tier names to import")
		for i from 1 to ntier
			select TextGrid 'name$'
			tier_name$ = Get tier name... i
			boolean("Tier'i' ('tier_name$')",1)
		endfor
	clicked = endPause("Cancel","Start",2,1)
	select TextGrid 'name$'
	Remove

	for i from 1 to numberOfFiles
		#Duplicate and remove the unselected tiers, if any
		select Strings FileList
		fileName$ = Get string... i
		name$ = fileName$ - ".wav" - ".WAV"
		Read from file... 'directory$'/'name$'.'ext$'
		select TextGrid 'name$'
		ntier = Get number of tiers
		start_time = Get start time
		end_time = Get end time
		Copy... 'name$'_imported
	
		for j from 1 to ntier
			k = ntier + 1 - j
			if tier'k' = 0
				select TextGrid 'name$'_imported
				Remove tier... 'k'
			endif
		endfor

		select TextGrid 'name$'_imported
		new_ntier = Get number of tiers

		left_edge = end_time
		#Synchronize the left edges
		for j from 1 to new_ntier
			t = Get end point... j 1
			if t < left_edge
				left_edge = t
			endif
		endfor
	
		for j from 1 to new_ntier
			label$ = Get label of interval... j 2
			Remove right boundary... j 1
			Insert boundary... j left_edge
			Set interval text... j 1 
			Set interval text... j 2 'label$'
		endfor
	
		right_edge = start_time
		#Synchronize the right edges
		for j from 1 to new_ntier
			k = Get number of intervals... j
			t = Get start point... j k
			if t > right_edge
				right_edge = t
			endif
		endfor
	
		for j from 1 to new_ntier
			k = Get number of intervals... j
			Remove left boundary... j k
			Insert boundary... j right_edge
		endfor
	
		printline 'name$' 'left_edge' 'right_edge'

		select TextGrid 'name$'_imported
		Write to short text file... 'directory$'/'name$'.annotation_short

		select TextGrid 'name$'
		plus TextGrid 'name$'_imported
		Remove
	endfor

	process_all_files_without_pause = 1
	select Strings extension
	Remove
	printline Your annotation files have been imported.
endproc

procedure Learn
	### Error Checking ###

	#[EXIT_L02] Cannot find configuration file (config.txt)
	if not fileReadable("'directory$'/config.txt")
		exit Error Code L02: Cannot find configuration file (config.txt)
	endif

	#[EXIT_L03] Cannot find function profile file (function_profile.txt)
	if not fileReadable("'directory$'/function_profile.txt")
		exit Error Code L03: Cannot find function profile file (function_profile.txt)
	endif

	Create Strings as file list... aList 'directory$'/*.annotation
	n_a = Get number of strings
	if n_a = 0
		#[EXIT_L04] Cannot find annotation files
		exit Error Code L04: Cannot find annotation files	
	elsif n_a < numberOfFiles
		#[EXIT_L05] Number of annotation files must equal number of sound files
		exit Error Code L05: Number of annotation files must equal number of sound files
	endif

	for i from 1 to n_a
		select Strings aList
		fileName$ = Get string... i
		name$ = fileName$ - ".annotation"
		nocheck Read from file... 'directory$'/'fileName$'
		fullName$ = selected$()
		name1$ = extractLine$(fullName$," ")
		if (name$ = name1$)
			Remove
		else
			#[EXIT_L06] Wrong format (annotation)
			exit Error Code L06: Wrong format ('name$'.annotation)
		endif
	endfor
	select Strings aList
	Remove

	Create Strings as file list... iList 'directory$'/*.intervals
	n_i = Get number of strings
	if n_i = 0
		#[EXIT_L07] Cannot find intervals files
		exit Error Code L07: Cannot find intervals files	
	elsif n_i < numberOfFiles
		#[EXIT_L08] Number of intervals files must equal number of sound files
		exit Error Code L08: Number of intervals files must equal number of sound files
	endif

	for i from 1 to n_i
		select Strings iList
		fileName$ = Get string... i
		name$ = fileName$ - ".intervals"
		nocheck Read from file... 'directory$'/'fileName$'
		fullName$ = selected$()
		name1$ = extractLine$(fullName$," ")
		if (name$ = name1$)
			Remove
		else
			#[EXIT_L09] Wrong format (intervals)
			exit Error Code L06: Wrong format ('name$'.intervals)
		endif
	endfor
	select Strings iList
	Remove

	Create Strings as file list... fList 'directory$'/*.actutimenormf0
	n_f = Get number of strings
	if n_f = 0
		#[EXIT_L10] Cannot find actutimenormf0 files
		exit Error Code L10: Cannot find actutimenormf0 files	
	elsif n_f < numberOfFiles
		#[EXIT_L11] Number of actutimenormf0 files must equal number of sound files
		exit Error Code L11: Number of actutimenormf0 files must equal number of sound files
	endif
	for i from 1 to n_f
		select Strings fList
		fileName$ = Get string... i
		name$ = fileName$ - ".actutimenormf0"
		nocheck Read from file... 'directory$'/'fileName$'
		fullName$ = selected$()
		name1$ = extractLine$(fullName$," ")
		if (name$ = name1$)
			Remove
		else
			#[EXIT_L12] Wrong format (actutimenormf0)
			exit Error Code L12: Wrong format ('name$'.actutimenormf0)
		endif
	endfor
	select Strings fList
	Remove

	Read from file... 'directory$'/config.txt
	c_nrows = Get number of rows
	if c_nrows <> 7
		#[EXIT_L13] Wrong format (config.txt)
		exit Error Code L13: Wrong format (config.txt)
	endif
	Remove

	Read from file... 'directory$'/function_profile.txt
	c_nrows = Get number of rows
	if c_nrows <> 3
		#[EXIT_L13] Wrong format (config.txt)
		exit Error Code L13: Wrong format (config.txt)
	endif
	Remove

	### End of Error Checking ###

	beginPause("Learn - Options")
		comment("Configure optimization parameters")
		integer("Maximum Iteration",700)
		real("Learning Rate",0.1)
		real("Starting Temperature", 700)
		real("Reduction Factor", 0.97)
		real("Silent Threshold",0.20)
		boolean("Inspect Manipulation",0)
	clicked = endPause("Cancel","Start",2,1)

	#[EXIT_00] Choose to not Continue
	if clicked = 1
		exit
	endif


	Create Strings as file list... IntervalList 'directory$'/*.intervals
	nintervals = Get number of strings
	Remove
	if !nintervals
		Create Strings as file list... AnnotationList 'directory$'/*.annotation
		n_annot = Get number of strings
		for i from 1 to n_annot
			Read from file... 'directory$'/'name$'.annotation
			Down to Table... no 4 yes no
			Save as tab-separated file... 'directory$'/'name$'.intervals
			select TextGrid 'name$'
			plus Table 'name$'
			Remove
		endfor
		select Strings AnnotationList
		Remove
	endif
	
	system java -jar Learn.jar "'directory$'" 'maximum_Iteration' 'learning_Rate' 'starting_Temperature' 'reduction_Factor' 'silent_Threshold'


	select Strings FileList
	numberOfFiles = Get number of strings
	Create Table with column names... accuracy numberOfFiles+1 Filename RMSE Correlation
	mean_rmse = 0
	mean_corr = 0
	for current_file from 1 to numberOfFiles
		select Strings FileList
		fileName$ = Get string... current_file
		name$ = fileName$ - ".wav" - ".WAV"
	
		Read from file... 'directory$'/'name$'.intervals
		start_time = Get value... 1 tmin
		select Table 'name$'
		Remove
	
		Read from file... 'directory$'/'name$'.synf0
		nrow = Get number of rows
		rel_end_time = Get value... nrow Time
		end_time = rel_end_time + start_time
		Create PitchTier... 'name$' start_time end_time
	
		sxy = 0
		sx = 0
		sy = 0
		sx2 = 0
		sy2 = 0
		sse = 0
		for i from 1 to nrow
			select Table 'name$'
			t_rel = Get value... i Time
			t = t_rel + start_time
			f0 = Get value... i F0syn
			orgf0 = Get value... i F0org
			f0hz = exp((1/12)*f0*ln(2))
			select PitchTier 'name$'
			Add point... 't' 'f0hz'
		
			sxy = sxy + f0*orgf0
			sx = sx + f0
			sy = sy + orgf0
			sx2 = sx2 + f0*f0
			sy2 = sy2 + orgf0*orgf0
		
			sse = sse + (f0-orgf0)*(f0-orgf0)
		endfor
	
		rmse = sqrt(sse/nrow)
		corr = (nrow*sxy - sx*sy)/(sqrt(nrow*sx2-sx*sx)*sqrt(nrow*sy2-sy*sy))
		mean_rmse += rmse
		mean_corr += corr
	
		select Table accuracy
		Set string value... current_file Filename 'name$'
		Set numeric value... current_file RMSE 'rmse'
		Set numeric value... current_file Correlation 'corr'
	
		if inspect_Manipulation
			Read from file... 'directory$'/'fileName$'
			To Manipulation... 0.01 75 500
			plus PitchTier 'name$'
			Replace pitch tier
		endif
	
		select Table 'name$'
		plus PitchTier 'name$'
		if inspect_Manipulation
			plus Sound 'name$'
		endif
		Remove
	endfor
	mean_rmse /= numberOfFiles
	mean_corr /= numberOfFiles

	select Table accuracy
	Set string value... numberOfFiles+1 Filename Mean
	Set numeric value... numberOfFiles+1 RMSE 'mean_rmse'
	Set numeric value... numberOfFiles+1 Correlation 'mean_corr'
	Save as tab-separated file... 'directory$'/accuracy_learning.txt
	Remove

	select Strings FileList
	Remove
	printline Target learning completed!
	printline mean RMSE = 'mean_rmse'		mean Correlation = 'mean_corr'
endproc

procedure Synthesize
	### Error Checking ###

	#[EXIT_S02] Cannot find the parameter file (parameters.txt)
	if not fileReadable("'directory$'/parameters.txt")
		exit Error Code L03: Cannot find the parameter file (parameters.txt)
	endif

	Create Strings as file list... aList 'directory$'/*.annotation
	n_a = Get number of strings
	if n_a = 0
		#[EXIT_S03] Cannot find annotation files
		exit Error Code L03: Cannot find annotation files	
	elsif n_a < numberOfFiles
		#[EXIT_S04] Number of annotation files must equal number of sound files
		exit Error Code S04: Number of annotation files must equal number of sound files
	endif
	for i from 1 to n_a
		select Strings aList
		fileName$ = Get string... i
		name$ = fileName$ - ".annotation"
		nocheck Read from file... 'directory$'/'fileName$'
		fullName$ = selected$()
		name1$ = extractLine$(fullName$," ")
		if (name$ = name1$)
			Down to Table... no 4 yes no
			Save as tab-separated file... 'directory$'/'name$'.intervals
		
			select TextGrid 'name$'
			plus Table 'name$'
			Remove
		else
			#[EXIT_S05] Wrong format (annotation)
			exit Error Code S05: Wrong format ('name$'.annotation)
		endif
	endfor
	select Strings aList
	Remove


	Create Strings as file list... iList 'directory$'/*.intervals
	n_i = Get number of strings
	if n_i = 0
		#[EXIT_S06] Cannot find intervals files
		exit Error Code S06: Cannot find intervals files	
	elsif n_i < numberOfFiles
		#[EXIT_S07] Number of intervals files must equal number of sound files
		exit Error Code S07: Number of intervals files must equal number of sound files
	endif
	for i from 1 to n_i
		select Strings iList
		fileName$ = Get string... i
		name$ = fileName$ - ".intervals"
		nocheck Read from file... 'directory$'/'fileName$'
		fullName$ = selected$()
		name1$ = extractLine$(fullName$," ")
		if (name$ = name1$)
			Remove
		else
			#[EXIT_S08] Wrong format (intervals)
			exit Error Code S08: Wrong format ('name$'.intervals)
		endif
	endfor
	select Strings iList
	Remove


	Create Strings as file list... fList 'directory$'/*.actutimenormf0
	n_f = Get number of strings
	if n_f = 0
		#[EXIT_S09] Cannot find actutimenormf0 files
		exit Error Code S09: Cannot find actutimenormf0 files	
	elsif n_f < numberOfFiles
		#[EXIT_S10] Number of actutimenormf0 files must equal number of sound files
		exit Error Code S10: Number of actutimenormf0 files must equal number of sound files
	endif
	for i from 1 to n_f
		select Strings fList
		fileName$ = Get string... i
		name$ = fileName$ - ".actutimenormf0"
		nocheck Read from file... 'directory$'/'fileName$'
		fullName$ = selected$()
		name1$ = extractLine$(fullName$," ")
		if (name$ = name1$)
			Remove
		else
			#[EXIT_S11] Wrong format (actutimenormf0)
			exit Error Code S11: Wrong format ('name$'.actutimenormf0)
		endif
	endfor
	select Strings fList
	Remove

	Create Strings as file list... sfList 'directory$'/*.semitonef0
	n_sf = Get number of strings
	if n_sf = 0
		#[EXIT_S12] Cannot find actutimenormf0 files
		exit Error Code S12: Cannot find actutimenormf0 files	
	elsif n_sf < numberOfFiles
		#[EXIT_S13] Number of actutimenormf0 files must equal number of sound files
		exit Error Code S13: Number of actutimenormf0 files must equal number of sound files
	endif
	for i from 1 to n_sf
		select Strings sfList
		fileName$ = Get string... i
		name$ = fileName$ - ".semitonef0"
		nocheck Read from file... 'directory$'/'fileName$'
		fullName$ = selected$()
		name1$ = extractLine$(fullName$," ")
		if (name$ = name1$)
			Remove
		else
			#[EXIT_S14] Wrong format (actutimenormf0)
			exit Error Code S14: Wrong format ('name$'.actutimenormf0)
		endif
	endfor
	select Strings sfList
	Remove

	### End of Error Checking ###

	beginPause("Synthesize - Options")
		integer("Input File No",1)
		choice("Synthesis_task:",1)
			option("Interactive view")
			option("Synthesize all sounds without pause")
		boolean("Inspect Manipulation",0)
		boolean("Save synthesized sound",0)
	clicked = endPause("Cancel","Start",2,1)

	#[EXIT_00] Choose to not Continue
	if clicked = 1
		exit
	endif

	if save_synthesized_sound = 1
		outputdir$ = chooseDirectory$("Choose the output directory")
		if outputdir$ = ""
			exit Output directory must be selected to write the synthesized sound
		endif
	endif

	max_strength = 100

	Create Table with column names... accuracy numberOfFiles+1 Filename RMSE Correlation

	mean_rmse = 0
	mean_corr = 0
	for current_file from input_File_No to numberOfFiles
		select Strings FileList
		fileName$ = Get string... current_file
		name$ = fileName$ - ".wav" - ".WAV"
		system java -jar Synthesize.jar "'directory$'" 'name$' parameters.txt
	
		Read from file... 'directory$'/'name$'.synthesizedf0
		nrow = Get number of rows
		start_time = Get value... 1 Time
		end_time = Get value... nrow Time
		Create PitchTier... 'name$'_syn start_time end_time
		for i from 1 to nrow
			select Table 'name$'
			t = Get value... i Time
			f0 = Get value... i F0
			select PitchTier 'name$'_syn
			Add point... 't' 'f0'
		endfor
	
		Read from file... 'directory$'/'name$'.syn_acc
		Rename... 'name$'_acc
		rmse = Get value... 1 RMSE
		corr = Get value... 1 CORR
		mean_rmse += rmse
		mean_corr += corr

		select Table accuracy
		Set string value... current_file Filename 'name$'
		Set numeric value... current_file RMSE 'rmse'
		Set numeric value... current_file Correlation 'corr'
	
		Read from file... 'directory$'/'fileName$'
		To Manipulation... 0.01 75 500
		plus PitchTier 'name$'_syn
		Replace pitch tier

		if save_synthesized_sound
			select Manipulation 'name$'
			Get resynthesis (overlap-add)
			Save as WAV file... 'outputdir$'/'name$'_syn.wav
			Remove
		endif
	
		select Table 'name$'
		plus Table 'name$'_acc
		Remove
	
		Read from file... 'directory$'/'name$'.PitchTier
		Read from file... 'directory$'/'name$'.annotation
		Read from file... 'directory$'/'name$'.targets
	
		if synthesis_task = 1
			select Sound 'name$'
			tInitial = Get start time
			tFinal = Get end time
			call plot_target tInitial tFinal
		endif
		select Sound 'name$'
		plus PitchTier 'name$'
		plus PitchTier 'name$'_syn
		if inspect_Manipulation = 0
			plus Manipulation 'name$'
		endif
		plus TextGrid 'name$'
		plus Table 'name$'
		Remove
	endfor
	mean_rmse /= numberOfFiles
	mean_corr /= numberOfFiles
endproc

procedure plot_target t1 t2
	demoWindowTitle ("PENTAtrainer 2")
	demo Erase all
	demo Paint rectangle... White 0 100 -100 100
	demo Blue
	demo Times
	demo Line width... 2
	demo 12
	demo Select outer viewport... 0 100 50 100
	select PitchTier 'name$'
	To Pitch... 0.02 60 600
	Formula... 12 * ln (self) / ln(2); semitone
	f0_floor = Get minimum... 0 0 Hertz Parabolic
	f0_ceiling = Get maximum... 0 0 Hertz Parabolic
	f0_floor -= 10
	f0_ceiling += 10
	demo Draw... t1 t2 f0_floor f0_ceiling no
	Remove
	demo Line width... 0.5
	demo Draw inner box

	select PitchTier 'name$'_syn
	Copy... 'name$'_synst
	Formula... 12 * ln (self) / ln(2); semitone
	demo Black
	demo Text special... t1-0.04*(t2-t1) Centre 0.5*(f0_floor+f0_ceiling) Bottom Times 12 90 Pitch (st)
	demo Text special... 0.5*(t1+t2) Centre f0_floor-0.125*(f0_ceiling-f0_floor) Top Times 12 0 Time (s)
	demo Red
	demo Draw... t1 t2 f0_floor f0_ceiling no
	Remove

	demo Marks bottom every... 1 0.1 no yes no
	demo Marks bottom every... 1 0.5 yes yes no
	demo Marks left every... 1 5 no yes no
	demo Marks left every... 1 10 yes yes no

	select TextGrid 'name$'
	nintervals = Get number of intervals... 1

	# Remove lines in voiceless regions
	last_end = 32767
	for nn from 1 to nintervals
		select TextGrid 'name$'
		label$ = Get label of interval... 1 nn
		if not label$ = ""
			start = Get starting point... 1 nn
			end = Get end point... 1 nn
			if (start - last_end) > 0.001
				demo Paint rectangle... White last_end start f0_floor+1 f0_ceiling-1
			endif
			last_end = end
		elsif nn = 1 or nn = nintervals
			start = Get starting point... 1 nn
			end = Get end point... 1 nn
			demo Paint rectangle... White start end-0.001 f0_floor+1 f0_ceiling-1
		endif
	endfor

	# Draw pitch targets and syllable boundaries
	interval_plot = 0
	last_end = 32767
	demo Dashed line
	for nn from 1 to nintervals
		select TextGrid 'name$'
		label$ = Get label of interval... 1 nn
		if not label$ = ""
			interval_plot += 1
			start = Get starting point... 1 nn
			end = Get end point... 1 nn
			duration = end - start
			select Table 'name$'
			m = Get value... 'interval_plot' m
			b = Get value... 'interval_plot' b
			lambda = Get value... 'interval_plot' r
			strength = if lambda > 0 then 3 * lambda / max_strength else 0.001 fi
			y1 = b - m*duration
			y2 = b
			# demo Black
			# demo Text special... start Left f0_ceiling Bottom Times 12 0 'label$'
			demo Green
			demo Line width... strength
			demo Draw line... start y1 end y2
			demo Grey
			demo Line width... 0.25
			demo Draw line... start f0_floor start f0_ceiling
			if (start - last_end) > 0.001
				demo Draw line... last_end f0_floor last_end f0_ceiling
			endif
			last_end = end
		elsif nn = 1 or nn = nintervals
			start = Get starting point... 1 nn
			demo Grey
			demo Line width... 0.25
			demo Draw line... start f0_floor start f0_ceiling
		endif
	endfor

	# Draw functional layers
	demo Navy
	demo Select outer viewport... 0 100 10 49
	demo Axes... t1 t2 0 10
	demo Text special... t2 right 10 bottom Courier 12 0 Communicative Functions
	demo Draw inner box

		# Draw syllable boundaries
		interval_plot = 0
		last_end = 32767
		demo Dashed line
		demo Grey
		for nn from 1 to nintervals
			select TextGrid 'name$'
			label$ = Get label of interval... 1 nn
			if not label$ = ""
				start = Get starting point... 1 nn
				end = Get end point... 1 nn
				demo Line width... 0.25
				demo Draw line... start 0 start 10
				if (start - last_end) > 0.001
					demo Draw line... last_end 0 last_end 10
				endif
				last_end = end
			elsif nn = 1 or nn = nintervals
				start = Get starting point... 1 nn
				demo Line width... 0.25
				demo Draw line... start 0 start 10
			endif
		endfor

		# Draw labels
		select TextGrid 'name$'
		ntiers = Get number of tiers
		top_level = 10
		for tt from 1 to ntiers
			tier_level = top_level - tt
			tier_name$ = Get tier name... tt
			for nn from 1 to nintervals
				select TextGrid 'name$'
				label$ = Get label of interval... tt nn
				if not label$ = ""
					start = Get starting point... tt nn
					end = Get end point... tt nn
					middle = (start+end)/2
					demo Black
					demo Text special... middle centre tier_level half Times 12 0 'label$'
				endif
			endfor
			demo Maroon
			demo Text special... t2 Right tier_level half Courier 12 0 'tier_name$'
		endfor
	
	# Draw interfaces
	demo Select inner viewport... 0 95 95 98
	demo Axes... 7 95 0 3
	demo Paint rounded rectangle... {0.98,0.98,0.8} 12 18 0 3 2	; zoom in
	demo Paint rounded rectangle... {0.98,0.98,0.8} 19 25 0 3 2	; zoom out
	demo Paint rounded rectangle... {0.98,0.98,0.8} 26 32 0 3 2	; zoom all
	demo Paint rounded rectangle... {0.98,0.98,0.8} 34 38 0 3 2	; shift to start
	demo Paint rounded rectangle... {0.98,0.98,0.8} 39 43 0 3 2	; shift left
	demo Paint rounded rectangle... {0.98,0.98,0.8} 44 48 0 3 2	; shif right
	demo Paint rounded rectangle... {0.98,0.98,0.8} 49 53 0 3 2	; shift to end
	demo Paint rounded rectangle... {0.98,0.98,0.8} 55 63 0 3 2	; play original
	demo Paint rounded rectangle... {0.98,0.98,0.8} 64 74 0 3 2	; play resynthesis
	demo Paint rounded rectangle... {0.98,0.98,0.8} 76 82 0 3 2	; Previous
	demo Paint rounded rectangle... {0.98,0.98,0.8} 83 87 0 3 2	; Next
	demo Paint rounded rectangle... {0.98,0.98,0.8} 89 93 0 3 2	; Exit
	demo Solid line
	demo Line width... 0.5
	demo Black
	demo Draw rounded rectangle... 12 18 0 3 2
	demo Draw rounded rectangle... 19 25 0 3 2
	demo Draw rounded rectangle... 26 32 0 3 2
	demo Draw rounded rectangle... 34 38 0 3 2
	demo Draw rounded rectangle... 39 43 0 3 2
	demo Draw rounded rectangle... 44 48 0 3 2
	demo Draw rounded rectangle... 49 53 0 3 2
	demo Draw rounded rectangle... 55 63 0 3 2
	demo Draw rounded rectangle... 64 74 0 3 2
	demo Draw rounded rectangle... 76 82 0 3 2
	demo Draw rounded rectangle... 83 87 0 3 2
	demo Draw rounded rectangle... 89 93 0 3 2
	demo Text special... 15 centre 1.5 half Times 10 0 Zoom in
	demo Text special... 22 centre 1.5 half Times 10 0 Zoom out
	demo Text special... 29 centre 1.5 half Times 10 0 Zoom all
	demo Text special... 36 centre 1.5 half Helvetica 10 0 |<<
	demo Text special... 41 centre 1.5 half Helvetica 10 0 <<
	demo Text special... 46 centre 1.5 half Helvetica 10 0 >>
	demo Text special... 51 centre 1.5 half Helvetica 10 0 >>|
	demo Text special... 59 centre 1.5 half Times 10 0 Play original
	demo Text special... 69 centre 1.5 half Times 10 0 Play resynthesis
	demo Text special... 79 centre 1.5 half Times 10 0 Previous
	demo Text special... 85 centre 1.5 half Times 10 0 Next
	demo Text special... 91 centre 1.5 half Times 10 0 Exit
	demo Select inner viewport... 0 50 0 10
	demo Axes... 0 50 0 10
	demo Text special... 2 left 9 bottom Courier 12 0 Current sound: 'name$' (No.'current_file')

	demo Select inner viewport... 50 100 0 10
	demo Axes... 0 50 0 10
	demo Black
	demo Text special... 20 left 10 bottom Helvetica 12 0 Legends

	demo Blue
	demo Line width... 2
	demo Draw line... 20 8 22 8
	demo Text special... 23 left 8 half Helvetica 10 0 Original F0 Contour

	demo Red
	demo Line width... 1
	demo Draw line... 20 6 22 6
	demo Text special... 23 left 6 half Helvetica 10 0 Synthesized F0 Contour

	demo Green
	demo Dashed line
	demo Line width... 1
	demo Draw line... 20 4 22 4
	demo Text special... 23 left 4 half Helvetica 10 0 Pitch Targets

	if more_file = 0
;		demo Paint rectangle... White 0 100 25 30
		demo Red
		demo Text special... 10 left 57 bottom Courier 18 0 Last sound file...
	elsif more_file < 0
;		demo Paint rectangle... White 0 100 25 30
		demo Red
		demo Text special... 10 left 57 bottom Courier 18 0 First sound file...
	endif
	call on_input
endproc

procedure on_input
	demo Select inner viewport... 0 95 95 98
	demo Axes... 7 95 0 3
	loop = 1
	while (loop and demoWaitForInput ( ))
		if demoClicked ( )
			if demoClickedIn (12, 18, 0, 3)			; clicked Zoom in
				new_window_centre = t1 + (t2-t1)/2
				plot_width = 0.5 * (t2-t1)
				t1 = max (new_window_centre - plot_width/2, tInitial)
				t2 = min (new_window_centre + plot_width/2, tFinal)
				plot_width = t2-t1
				call plot_target t1 t2
			elsif demoClickedIn (19, 25, 0, 3)		; clicked Zoom out
				new_window_centre = t1 + (t2-t1)/2
				plot_width = 2 * (t2-t1)
				t1 = max (new_window_centre - plot_width/2, tInitial)
				t2 = min (new_window_centre + plot_width/2, tFinal)
				plot_width = t2-t1
				call plot_target t1 t2
			elsif demoClickedIn (26, 32, 0, 3)		; clicked Zoom all
				new_window_centre = tInitial + (tFinal-tInitial)/2
				plot_width = 100 * (tFinal-tInitial)
				t1 = max (new_window_centre - plot_width/2, tInitial)
				t2 = min (new_window_centre + plot_width/2, tFinal)
				plot_width = t2-t1
				call plot_target t1 t2
			elsif demoClickedIn (34, 38, 0, 3)			; clicked Shift to start
				call plot_target tInitial t1+plot_width
			elsif demoClickedIn (39, 43, 0, 3)		; clicked Shift left
				t1 = min (t1 - plot_width, tFinal + plot_width)
				t1 = max (t1, tInitial)
				t2 = t1 + plot_width
				call plot_target t1 t2
			elsif demoClickedIn (44, 48, 0, 3)		; clicked Shift right
				t1 = min (t1 + plot_width, tFinal - plot_width)
				t1 = max (t1, tInitial)
				t2 = t1 + plot_width
				call plot_target t1 t2
			elsif demoClickedIn (49, 53, 0, 3)		; clicked Shift to end
				call plot_target tFinal-plot_width tFinal
			elsif demoClickedIn (55, 63, 0, 3)		; clicked Play original
				demo Paint rounded rectangle... Yellow 55 63 0 3 2
				demo Black
				demo Draw rounded rectangle... 55 63 0 3 2
				demo Text special... 59 centre 1.5 half Times 10 0 Play original
				select Sound 'name$'
				Extract part... t1 t2 Rectangular 1.0 yes
				Play
				Remove
				demo Paint rounded rectangle... {0.98,0.98,0.8} 55 63 0 3 2
				demo Black
				demo Draw rounded rectangle... 55 63 0 3 2
				demo Text special... 59 centre 1.5 half Times 10 0 Play original
			elsif demoClickedIn (64, 74, 0, 3)		; clicked Play resynthesis
				# if found_interval
					demo Paint rounded rectangle... Yellow 64 74 0 3 2
					demo Black
					demo Draw rounded rectangle... 64 74 0 3 2
					demo Text special... 69 centre 1.5 half Times 10 0 Play resynthesis
				
					select Manipulation 'name$'
					plus PitchTier 'name$'_syn
					Replace pitch tier
				
					select Manipulation 'name$'
					resyn = Get resynthesis (overlap-add)
					Extract part... t1 t2 Rectangular 1.0 yes
					Play
					plus resyn
					Remove
					# plus sound
					# plus manipulation
				
					demo Paint rounded rectangle... {0.98,0.98,0.8} 64 74 0 3 2
					demo Black
					demo Draw rounded rectangle... 64 74 0 3 2
					demo Text special... 69 centre 1.5 half Times 10 0 Play resynthesis
				# else
					# demo Paint rectangle... White 0 100 25 30
					# demo Red
					# demo Text special... 2 left 27 bottom Courier 14 0 No resynthesis...
				# endif
			elsif demoClickedIn (76, 82, 0, 3)		; clicked Previous
				if current_file < 2
					more_file -= 2
					call plot_target t1 t2
				else
					current_file -= 2
					demo Erase all
					loop = 0
					replot = 0
					more_file = 1
				endif
			elsif demoClickedIn (83, 87, 0, 3)		; clicked Next
				if current_file = numberOfFiles
					more_file = 0
					call plot_target t1 t2
				else
					demo Erase all
					loop = 0
					replot = 0
					more_file = 1
				endif				
			elsif demoClickedIn (89, 93, 0, 3)		; clicked Exit
				demo Select inner viewport... 0 100 50 100
				demo Axes... 0 100 0 2
				demo Red
				demo Text special... 30 left 0.5 bottom Courier 18 0 Now close window manually...
				loop = 0
				select Sound 'name$'
				plus PitchTier 'name$'
				plus PitchTier 'name$'_syn
				plus Manipulation 'name$'
				plus TextGrid 'name$'
				plus Table 'name$'
				Remove
;					goto EXIT
				exit
			endif
		endif
		demo Select inner viewport... 0 95 95 98
		demo Axes... 7 95 0 3
	endwhile
endproc
